/* * * * * * * * * * * * * * * * * * * * * * * *
 * SCIDYNE Corporation, Oct 14 2021  Mark Durgin 
 * Analog Input demo program for MULTI-IO-ARD PN #100-7714
 * 
 * Display Single-Ended conversions of Analog Inputs Channels 0 - 7
 * Results displayed on Arduino IDE Serial Monitor.
 * 
 * For compatibility with 3.3V and 5V hardware this program sets the
 * Analog-to-Digital Converter input range for all Ain channels to +/- 2.048 volts.
 * See ADS1115 datasheet for details of setup and operation
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Important include files
 */
#include <Wire.h>   // Needed for Arduino I2C support.  

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Prototypes
 */
uint16_t readADC (unsigned char channel);    // Digitize an analog input channel, return 16-bit result

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Program constants and defines
 */
#define COMPORT Serial                       // Serial Port to use: UNO, MEGA use Serial,  DUE may use Serial1
#define WIREPORT Wire                        // Set Wire Library: UNO, MEGA use Wire, DUE use Wire1

#define ADS1115_0_ADDRESS   (0x48)           // I2C Address of first ADS1115, AICH0 - AICH3
#define ADS1115_1_ADDRESS   (0x49)           // I2C Address of second ADS1115, AICH4 - AICH7

#define ADS1X15_REG_POINTER_CONVERT (0x00)   // ADS1115 internal Conversion Register 
#define ADS1X15_REG_POINTER_CONFIG  (0x01)   // ADS1115 internal Configuration Register

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * setup runs once then execution goes to loop
 */
void setup()
{
  WIREPORT.begin();                         // Setup I2C communications

  COMPORT.begin(9600);                  // Setup communications to IDE Monitor
    while (!COMPORT);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * loop runs continuously
 * Digitize each Analog Input channel and display results on 
 * Arduino IDE Serial Monitor.
 */
void loop()
{
   uint16_t i;        // loop and channel variable
   int16_t result;    // Holds 16-bit ADC conversion result
  
   for (i = 0; i < 8; i++)
   {
       result = readADC(i);
       COMPORT.print("Channel-");
       COMPORT.print(i);
       COMPORT.print(": ");
       COMPORT.println(result,DEC);
   }
   
   COMPORT.println("-----------------");  
   delay(1000);
}


/* * * * * * * * * * * * * * * * * * * * * * * * *
 * Read an ADC Channel
 * Call with desired channel, Returns signed 16-bit result
 */
uint16_t readADC (unsigned char AIchannel)
{
  int16_t var = 0x0000;            // 16-bit temp storage variable
  uint16_t config_var;              // Used to build configuration register bits
  uint8_t ads1115_address;          // Holds I2C address of ADS115 being accessed

  // Assure AIchannel is within valid range of 0-7
  AIchannel &= 0x07;  
  
  // Determine which ADS1115 chip will be used
  ads1115_address = (AIchannel > 3) ? ADS1115_1_ADDRESS : ADS1115_0_ADDRESS;
  
  // Build the 16-bit ADS1115 Configuration Register, see ADS1115 data sheet for details
  config_var =
      (0x0003)    |         // Bit 0..1     Disable the comparator (default val)
      (0x0000)    |         // Bit 2        Non-latching (default val)
      (0x0000)    |         // Bit 3        Alert/Rdy active low   (default val)
      (0x0000)    |         // Bit 4        Traditional comparator (default val)
      (0x0080)    |         // Bit 5..7     Samples-per-second rate, 128 samples-per-second (default)
      (0x0100)    |         // Bit 8        Single-shot mode (default)
      (0x0400)    |         // Bit 9..11    Set input voltage range, default is +/- 2.048 Volts
      /* Set below */       // Bit 12..13   Mux Channel
      (0x4000)              // Bit-14       Single-Ended / Differential Mode
      /* Set below */   ;   // Bit 15       Write: Single-Shot start, Read: Conversion status
  
  // Setup for conversion on desired input channel where bits 12 and 13 set the mux channel 0-3
  config_var |= (AIchannel % 4) << 12; 

  // Set bit 15 to 'start single-conversion' when written
  config_var |= 0x8000;
  
  // Write config register to the ADC which starts the conversion
  WIREPORT.beginTransmission(ads1115_address);       // Open I2C communication with the ADS1115
  WIREPORT.write(ADS1X15_REG_POINTER_CONFIG);        // Next 2 bytes intended for config register
  WIREPORT.write(config_var >> 8);                   // Write config register High-Byte
  WIREPORT.write(config_var & 0xFF);                 // Write Config register Low-Byte
  WIREPORT.endTransmission();                        // Close I2C communications
  
  // Wait for the conversion to complete
  do {
    WIREPORT.requestFrom((uint8_t) ads1115_address, (uint8_t) 2);        // Request 2 bytes (16-Bits) from the ADS1115
    var = WIREPORT.read() << 8;                    // Read the configuration register High-Byte
    var |= WIREPORT.read();                        // Read the configuration register Low-Byte
  } while (!(var & 0x8000));                   // Loop until bit 15 is "0", i.e.; conversion complete
  
  // Read the signed 16-bit conversion results
  // Concatenate the two bytes to form the 16-bit result
  WIREPORT.beginTransmission(ads1115_address);     // Open I2C communication with the ADS1115
  WIREPORT.write(ADS1X15_REG_POINTER_CONVERT);     // Next 2 bytes read will be from conversion register
  WIREPORT.endTransmission();                      // Close I2C communications
  
  WIREPORT.requestFrom((uint8_t) ads1115_address, (uint8_t) 2);        // Request 2 bytes (16-Bits) from the ADS1115
  var = WIREPORT.read() << 8;                      // First byte is High-Byte, bits [15:8] 8..15
  var |= WIREPORT.read();                          // Second byte is Low-Byte, bits [7:0]  0..7
  
  return(var);                                 // Return the result
  
}
 
